@@ -9,7 +9,7 @@
 #
 package MooseX::AttributeShortcuts;
 {
-  $MooseX::AttributeShortcuts::VERSION = '0.012'; # TRIAL
+  $MooseX::AttributeShortcuts::VERSION = '0.016'; # TRIAL
 }
 
 # ABSTRACT: Shorthand for common attribute options
@@ -29,7 +29,7 @@ use Moose::Util::MetaRole;
 {
     package MooseX::AttributeShortcuts::Trait::Attribute;
 {
-  $MooseX::AttributeShortcuts::Trait::Attribute::VERSION = '0.012'; # TRIAL
+  $MooseX::AttributeShortcuts::Trait::Attribute::VERSION = '0.016'; # TRIAL
 }
     use namespace::autoclean;
     use MooseX::Role::Parameterized;
@@ -57,11 +57,22 @@ use Moose::Util::MetaRole;
             clearer   => 'clear',
             trigger   => '_trigger_',
             %{ $p->prefixes },
-       );
+        );
+
+        has anon_builder => (
+            reader    => 'anon_builder',
+            writer    => '_set_anon_builder',
+            isa       => 'CodeRef',
+            predicate => 'has_anon_builder',
+            init_arg  => '_anon_builder',
+        );
 
         my $_process_options = sub {
             my ($class, $name, $options) = @_;
 
+            my $_has = sub { defined $options->{$_[0]} };
+            my $_opt = sub { $_has->(@_) ? $options->{$_[0]} : q{} };
+
             if ($options->{is}) {
 
                 if ($options->{is} eq 'rwp') {
@@ -75,7 +86,7 @@ use Moose::Util::MetaRole;
                     $options->{is}       = 'ro';
                     $options->{lazy}     = 1;
                     $options->{builder}  = 1
-                        unless $options->{builder} || $options->{default};
+                        unless $_has->('builder') || $_has->('default');
                 }
             }
 
@@ -84,27 +95,39 @@ use Moose::Util::MetaRole;
                 $options->{lazy_build} = 1;
                 $options->{clearer}    = "_clear_$name";
                 $options->{predicate}  = "_has_$name";
-                $options->{init_arg}   = "_$name" unless exists $options->{init_arg};
             }
 
             my $is_private = sub { $name =~ /^_/ ? $_[0] : $_[1] };
             my $default_for = sub {
                 my ($opt) = @_;
 
-                if ($options->{$opt} && $options->{$opt} eq '1') {
-                    $options->{$opt} =
-                        $is_private->('_', q{}) .
-                        $prefix{$opt} .
-                        $is_private->(q{}, '_') .
-                        $name;
-                }
+                return unless $_has->($opt);
+                my $opt_val = $_opt->($opt);
+
+                my ($head, $mid)
+                    = $opt_val eq '1'  ? ($is_private->('_', q{}), $is_private->(q{}, '_'))
+                    : $opt_val eq '-1' ? ($is_private->(q{}, '_'), $is_private->(q{}, '_'))
+                    :                    return;
+
+                $options->{$opt} = $head . $prefix{$opt} . $mid . $name;
                 return;
             };
 
+            # XXX install builder here if a coderef
+            if (defined $options->{builder}) {
+
+                #if (ref $_opt->('builder') eq 'CODE') {
+                if ((ref $options->{builder} || q{}) eq 'CODE') {
+
+                    $options->{_anon_builder} = $options->{builder};
+                    $options->{builder}       = 1;
+                }
+
+                $options->{builder} = "$bprefix$name"
+                    if $options->{builder} eq '1';
+            }
             ### set our other defaults, if requested...
             $default_for->($_) for qw{ predicate clearer };
-            $options->{builder} = "$bprefix$name"
-                if $options->{builder} && $options->{builder} eq '1';
             my $trigger = "$prefix{trigger}$name";
             $options->{trigger} = sub { shift->$trigger(@_) }
                 if $options->{trigger} && $options->{trigger} eq '1';
@@ -131,6 +154,21 @@ use Moose::Util::MetaRole;
 
             return $self->$orig($name, %options);
         };
+
+
+        # we hijack attach_to_class in order to install our anon_builder, if
+        # we have one.  Note that we don't go the normal
+        # associate_method/install_accessor/etc route as this is kinda...
+        # different.
+
+        after attach_to_class => sub {
+            my ($self, $class) = @_;
+
+            return unless $self->has_anon_builder;
+
+            $class->add_method($self->builder => $self->anon_builder);
+            return;
+        };
     };
 }
 
@@ -193,19 +231,21 @@ sub init_meta {
 
 1;
 
-
+__END__
 
 =pod
 
 =encoding utf-8
 
+=for :stopwords Chris Weyl
+
 =head1 NAME
 
 MooseX::AttributeShortcuts - Shorthand for common attribute options
 
 =head1 VERSION
 
-This document describes version 0.012 of MooseX::AttributeShortcuts - released May 02, 2012 as part of MooseX-AttributeShortcuts.
+This document describes version 0.016 of MooseX::AttributeShortcuts - released September 08, 2012 as part of MooseX-AttributeShortcuts.
 
 =head1 SYNOPSIS
 
@@ -297,7 +337,8 @@ Unless specified here, all options defined by L<Moose::Meta::Attribute> and
 L<Class::MOP::Attribute> remain unchanged.
 
 Want to see additional options?  Ask, or better yet, fork on GitHub and send
-a pull request.
+a pull request. If the shortcuts you're asking for already exist in L<Moo> or
+L<Mouse> or elsewhere, please note that as it will carry significant weight.
 
 For the following, "$name" should be read as the attribute name; and the
 various prefixes should be read using the defaults.
@@ -382,6 +423,21 @@ For an attribute named "_foo":
 This naming scheme, in which the trigger is always private, is the same as the
 builder naming scheme (just with a different prefix).
 
+=head2 builder => sub { ... }
+
+Passing a coderef to builder will cause that coderef to be installed in the
+class this attribute is associated with the name you'd expect, and
+C<builder =E<gt> 1> to be set.
+
+e.g., in your class,
+
+    has foo => (is => 'ro', builder => sub { 'bar!' });
+
+...is effectively the same as...
+
+    has foo => (is => 'ro', builder => '_build_foo');
+    sub _build_foo { 'bar!' }
+
 =for Pod::Coverage init_meta
 
 =head1 SOURCE
@@ -411,7 +467,3 @@ This is free software, licensed under:
   The GNU Lesser General Public License, Version 2.1, February 1999
 
 =cut
-
-
-__END__
-
